/* 
**  mod_spam_die.c -- Apache sample spam_die module
**  [Autogenerated via ``apxs -n spam_die -g'']
**
**  To play with this sample module first compile it into a
**  DSO file and install it into Apache's modules directory 
**  by running:
**
**    $ apxs -c -i mod_spam_die.c
**
**  Then activate it in Apache's httpd.conf file for instance
**  for the URL /spam_die in as follows:
**
**    #   httpd.conf
**    LoadModule spam_die_module modules/mod_spam_die.so
**    <Location /spam_die>
**    SetHandler spam_die
**    </Location>
**
**  Then after restarting Apache via
**
**    $ apachectl restart
**
**  you immediately can request the URL /spam_die and watch for the
**  output of this module. This can be achieved for instance via:
**
**    $ lynx -mime_header http://localhost/spam_die 
**
**  The output should be similar to the following one:
**
**    HTTP/1.1 200 OK
**    Date: Tue, 31 Mar 1998 14:42:22 GMT
**    Server: Apache/1.3.4 (Unix)
**    Connection: close
**    Content-Type: text/html
**  
**    The sample page from mod_spam_die.c
*/ 

#include <stdio.h>
#include <stdlib.h>

#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"

#define MAX_WORD 20

/* Initialize random seed */
static int spam_die_init_rand() {
    srand(apr_time_now());
}


/* Generate random number between st and end */
static int spam_die_rand(int st, int end) {
    int diff = end-st;
    
    if (diff < 0)
	diff = -diff;

    return (int)(rand()%diff)+st;
}

 
/* Generate random word */
static void spam_die_pr_word(request_rec *r, int add_sp) {
    int i, maxi;
    char buf[MAX_WORD+1];
    
    maxi = spam_die_rand(1, MAX_WORD-1);

    for (i=0; i<maxi;i++) {
	buf[i] = spam_die_rand('a', 'z');
    }
    
    if (add_sp==1) 
	buf[i++] = ' ';

    buf[i] = 0;

    ap_rputs(buf, r);
} 

static void spam_die_pr_para(request_rec *r) {
    /* 10 to 20 sentences */
    int i, j, seni = spam_die_rand(5, 10);
    
    for (j=0; j< seni; j++) {
	int maxi = spam_die_rand(5,15);
	for (i=0; i<maxi; i++) 
	    spam_die_pr_word(r, 1);
	
	ap_rputs(".", r);
    }
}

/* The sample content handler */
static int spam_die_handler(request_rec *r)
{
    int headi, maxi, i;
    char *prev_name = r->uri;

    if (strcmp(r->handler, "spam_die")) {
        return DECLINED;
    }
    r->content_type = "text/html";      

    if (r->header_only)
	return OK;

    ap_rputs("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"\n"
	     "\"http://www.w3.org/TR/REC-html40/loose.dtd\">\n", r);

    ap_rputs("<HTML>\n"
	     "<HEAD>\n"
	     "<TITLE>", r);

    maxi = spam_die_rand(3,7);
    for (i=0; i<maxi; i++) 
	spam_die_pr_word(r, 1);
       
    ap_rputs("</TITLE>\n"
	     "</HEAD>\n"
	     "<BODY>\n", r);

    /* Print Random title */
    headi = spam_die_rand(1,4);
    ap_rprintf(r, "<H%d>", headi);
    maxi = spam_die_rand(3,7);
    for (i=0; i<maxi; i++) 
	spam_die_pr_word(r, 1);
    ap_rprintf(r, "</H%d>\n", headi);
    
    spam_die_pr_para(r);

    if (strstr(r->uri, ".html")) {
	prev_name = apr_pstrdup(r->pool, r->uri);
	prev_name[strlen(prev_name)-5] = 0;
    } else {
	prev_name = apr_pstrcat(r->pool, r->uri, "/", NULL); 
    }

    for( i = 'a'; i<='z'; i++) {
	ap_rprintf(r, "[<A HREF=\"%s%c.html\">", 
		   prev_name, i);
	spam_die_pr_word(r, 0);
	ap_rprintf(r, "</A>] ");	
    }

    ap_rflush(r);

    /* Sleep 1 sec */
    apr_sleep(1500000);

    ap_rputs("<P>Please Get this emails:", r);
    
    maxi = spam_die_rand(10,20);
    for (i=0; i<maxi; i++) { 
	ap_rputs("<LI> <A HREF=\"mailto:", r);
	spam_die_pr_word(r, 0);
	ap_rputs("@", r);
	spam_die_pr_word(r, 0);
	ap_rputs(".", r);
	spam_die_pr_word(r, 0);
	ap_rputs(".com", r);
	ap_rputs("\">", r);
	spam_die_pr_word(r, 1);
	spam_die_pr_word(r, 1);
	ap_rputs("</A>\n", r);
    }
    
    spam_die_pr_para(r);

    ap_rputs("<BR><A href=mailto:listme@listme.dsbl.org>listme@listme.dsbl.org</A>", r);

    ap_rputs("<P><HR><I>Powered by ", r);
    spam_die_pr_word(r, 1);
    ap_rputs("<A HREF=\"http://spam-die.sf.net\">", r);
    spam_die_pr_word(r, 0);
    ap_rputs(", mod_spam_die, ", r);
    spam_die_pr_word(r, 0);
    ap_rputs("</A>", r); 
    
    ap_rputs("</BODY>\n"
	     "<HTML>", r);

    return OK;
}

static void spam_die_register_hooks(apr_pool_t *p)
{
    ap_hook_handler(spam_die_handler, NULL, NULL, APR_HOOK_MIDDLE);
}

/* Dispatch list for API hooks */
module AP_MODULE_DECLARE_DATA spam_die_module = {
    STANDARD20_MODULE_STUFF, 
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    spam_die_register_hooks  /* register hooks                      */
};

